<?php declare(strict_types=1);
/**
 * Badge Manager Class
 *
 * Manages the display of badges in WordPress plugin listings.
 *
 * @package wp-fail2ban-lib-badges
 * @since   1.0.0
 */

namespace WP_fail2ban\Lib\Badges;

use WP_fail2ban\Lib\Badges\Badge\LTSBadge;
use WP_fail2ban\Lib\Badges\Badge\CanonicalBadge;
use WP_fail2ban\Lib\Badges\Badge\NonCanonicalBadge;

defined('ABSPATH') or exit;

/**
 * Manages badge display in WordPress plugin listings
 *
 * @phpstan-type BadgeConfigType array{
 *     lts?: bool,
 *     canonical?: bool,
 *     free?: bool,
 *     show?: array{
 *         non-canonical?: bool
 *     }
 * }
 *
 * The BadgeConfig type defines the configuration options for badge display:
 * - lts: When true, displays the Long Term Support (LTS) badge
 * - canonical: When true, displays the Canonical version badge
 * - free: When true, displays the Free/Non-canonical version badge
 * - show: An array of options to control the display of badges:
 *   - non-canonical: When true, displays the Free/Non-canonical version badge
 *
 * All fields are optional and default to false when not specified.
 */
class BadgeManager
{
    /** 
     * @var BadgeConfigType
     * @since 1.0.0 
     */
    protected array $config;

    /**
     * Initializes the badge manager with the provided configuration
     *
     * Sets up the configuration and adds the necessary WordPress filter for plugin row meta.
     *
     * @since  1.0.0
     * @api
     * @param  BadgeConfigType    $config          Configuration options for badges
     * @throws void
     */
    public function __construct(array $config)
    {
        $this->config = $config;
        add_filter('plugin_row_meta', [$this, 'filterPluginRowMeta'], PHP_INT_MAX, 4);
    }

    /**
     * Static initializer for BadgeManager
     *
     * @since  1.0.0
     * @api
     * @param  BadgeConfigType  $config  Configuration options for badges
     * @return self                      A new BadgeManager instance
     */
    public static function init(array $config): self
    {
        // Assume no autoloader
        require_once __DIR__.'/Badge/AbstractBadge.php';
        require_once __DIR__.'/Badge/CanonicalBadge.php';
        require_once __DIR__.'/Badge/LTSBadge.php';
        require_once __DIR__.'/Badge/NonCanonicalBadge.php';

        return new self($config);
    }

    /**
     * Generates badge links based on configuration
     * 
     * Creates and renders the appropriate badge objects based on the current
     * configuration settings, combining them into a single HTML string.
     * 
     * @since 1.0.0
     * @throws void
     * @return string Combined HTML string of all configured badges
     */
    protected function generateBadgeLinks(): string
    {
        $links = [];

        if ($this->config['lts'] ?? false) {
            $badge = new LTSBadge();
            $links[] = $badge->render();
        }

        if ($this->config['canonical'] ?? false) {
            $badge = new CanonicalBadge();
            $links[] = $badge->render();
        }

        if ($this->config['free'] ?? false) {
            if ($this->config['show']['non-canonical'] ?? true) {
                $badge = new NonCanonicalBadge();
                $links[] = $badge->render();
            }
        }

        return implode('&nbsp;&nbsp;', $links);
    }

    /**
     * Filter callback for 'plugin_row_meta'
     *
     * Adds badge indicators to the plugin row in the WordPress admin.
     * Only modifies the meta for wp-fail2ban.php plugin file.
     *
     * @since  1.0.0
     * @param  array<string>     $links            Array of plugin row meta links
     * @param  string            $plugin_file_name The plugin file name
     * @param  array<mixed>      $plugin_data      Plugin data
     * @param  string            $status           Plugin status
     * @throws void
     * @return array<string>                       Modified array of plugin row meta links
     */
    public function filterPluginRowMeta($links, $plugin_file_name, $plugin_data, $status): array
    {
        if (0 === substr_compare($plugin_file_name, 'wp-fail2ban.php', -15, 15)) {
            $links[] = $this->generateBadgeLinks();
        }

        return $links;
    }
}